#  ----------------------------------------------------------------
# Copyright 2016 Cisco Systems
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ------------------------------------------------------------------

from ydk.ext.entity_utils import get_data_node_from_entity
from ydk.ext.entity_utils import get_entity_from_data_node
from ydk.ext.entity_utils import XmlSubtreeCodec

from .entity_utils import _datanode_to_entity, _read_entities, _payload_to_top_entity, _get_bundle_name

__all__ = [ "get_data_node_from_entity",
            "get_entity_from_data_node",
            "_get_bundle_name",
            "_datanode_to_entity",
            "_payload_to_top_entity",
            "_read_entities",
            "XmlSubtreeCodec"]



#  ----------------------------------------------------------------
# Copyright 2016 Cisco Systems
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ------------------------------------------------------------------
from ydk.ext.services import ExecutorService as _ExecutorService
from ydk.errors import YServiceError as _YServiceError
from ydk.errors.error_handler import handle_runtime_error as _handle_error


class ExecutorService(_ExecutorService):
    """ Python wrapper for ExecutorService
    """
    def __init__(self):
        self._es = _ExecutorService()

    def execute_rpc(self, provider, entity, top_entity=None):
        if None in (provider, entity):
            raise _YServiceError("provider and entity cannot be None")

        with _handle_error():
            return self._es.execute_rpc(provider, entity, top_entity)



#  ----------------------------------------------------------------
# Copyright 2016 Cisco Systems
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ------------------------------------------------------------------
import logging
import pkgutil
import importlib

from ydk import models
from ydk.types import EncodingFormat
from ydk.errors import YServiceProviderError
from ydk.path import Capability as _Capability
from ydk.path import Repository as _Repository


_TRACE_LEVEL_NUM = 5
_USER_PROVIDED_REPO = "ydk-user-provider-repo"


class CodecServiceProvider(object):
    """Python CodecServiceProvider wrapper.

    Args:
        type (ydk.types.EncodingFormat or str): Codec encoding format.
            Currently support XML or JSON.
        repo (ydk.path.Repository, optional): A user provided repository.
    Attributes:
        logger (logging.Logger): CodecServiceProvider logger.
        encoding (ydk.types.EncodingFormat): Codec encoding format.
        _root_schema_table (dict(str, ydk.path.RootSchemaNode)): A dictionary
            of root schemas, key is bundle name or _USER_PROVIDED_REPO, value
            is corresponding repository.
    """

    def __init__(self, **kwargs):
        self.logger = logging.getLogger(__name__)
        self._root_schema_table = {}

        repo = kwargs.get('repo', None)
        if repo is None:
            self._user_provided_repo = False
            self._repo = None
        else:
            self._user_provided_repo = True
            self._repo = repo

        encoding = kwargs.get('type', EncodingFormat.XML)
        if isinstance(encoding, str) and encoding == 'xml':
            encoding = EncodingFormat.XML
        elif isinstance(encoding, str) and encoding == 'json':
            encoding = EncodingFormat.JSON
        encoding = kwargs.get('encoding', encoding)
        self.encoding = encoding

    def initialize(self, bundle_name, models_path):
        """Initialize provider with bundle name and path for local YANG models.

        Args:
            bundle_name (str): bundle name.
            models_path (str): location for local YANG models.
        """
        if self._user_provided_repo:
            self._initialize_root_schema(bundle_name, self._repo, True)

        if bundle_name in self._root_schema_table:
            return

        self.logger.log(_TRACE_LEVEL_NUM, "Creating repo in path {}".format(models_path))
        repo = _Repository(models_path)
        self._initialize_root_schema(bundle_name, repo)

    def get_root_schema(self, bundle_name):
        """Return root_schema for bundle_name.

        Args:
            bundle_name (str): bundle name.
        """
        if self._user_provided_repo:
            return self._root_schema_table[_USER_PROVIDED_REPO]

        if bundle_name not in self._root_schema_table:
            self.logger.error("Root schema not created")
            raise YServiceProviderError(error_msg="Root schema not created")

        return self._root_schema_table[bundle_name]

    def _initialize_root_schema(self, bundle_name, repo, user_provided_repo=False):
        """Update root schema table entry.

        Args:
            name (str): bundle name.
            repo (ydk.path.Repository): default repository or repository provided by the user.
            user_provided_repo (bool, optional): Defaults to False.

        """
        name = bundle_name if not user_provided_repo else _USER_PROVIDED_REPO
        self.logger.log(_TRACE_LEVEL_NUM, "Initializing root schema for {}".format(name))
        # TODO: turn on and off libyang logging
        capabilities = []
        lookup_tables = self._get_bundle_capability_lookup_table(bundle_name)
        self._root_schema_table[name] = repo.create_root_schema(lookup_tables, capabilities)

    def _get_bundle_yang_ns(self, bundle_name):
        """Search installed local ydk-models python packages, and return _yang_ns

        Args:
            bundle_name (str): bundle name.

        Returns:
            mod_yang_ns (module): bundle's _yang_ns module.
        """
        mod_yang_ns = None
        for (_, name, ispkg) in pkgutil.iter_modules(models.__path__):
            if ispkg and name == bundle_name:
                try:
                    mod_yang_ns = importlib.import_module('ydk.models.{}._yang_ns'.format(name))
                    break
                except ImportError:
                    continue

        return mod_yang_ns

    def _get_bundle_capability_lookup_table(self, bundle_name):
        """Search installed local ydk-models python packages, and return corresponding
        capability lookup tables.

        Args:
            bundle_name (str): bundle name.

        Returns:
            lookup_tables(list of dict(str, ydk.types.Capability)): Capability lookup tables
        """
        name_namespace_lookup = {}

        mod_yang_ns = self._get_bundle_yang_ns(bundle_name)
        if mod_yang_ns is not None:
            capability_map = mod_yang_ns.__dict__['CAPABILITIES']
            namespace_map = mod_yang_ns.__dict__['NAMESPACE_LOOKUP']
            for d in (capability_map, namespace_map):
                name_namespace_lookup.update(d)

            for name in capability_map:
                cap = _Capability(name, capability_map[name])
                name_namespace_lookup[name] = cap
                # submodule
                if name in namespace_map:
                    name_namespace_lookup[namespace_map[name]] = cap

        return name_namespace_lookup

    def _get_bundle_capabilities(self, bundle_name):
        """Search installed local ydk-models python packages, and return corresponding
        capabilities.

        Args:
            bundle_name (str): bundle name.

        Returns:
            capabilities (list): List of ydk.path.Capability available for this bundle.
        """
        capabilities = []
        capability_map = {}

        mod_yang_ns = self._get_bundle_yang_ns(bundle_name)
        if mod_yang_ns is not None:
            capability_map = mod_yang_ns.__dict__['CAPABILITIES']
            for name in capability_map:
                capabilities.append(_Capability(name, capability_map[name]))

        return capabilities



#  ----------------------------------------------------------------
# Copyright 2016 Cisco Systems
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ------------------------------------------------------------------
from ydk.ext.services import CRUDService as _CrudService
from ydk.errors.error_handler import handle_runtime_error as _handle_error
from ydk.errors.error_handler import check_argument as _check_argument
from ydk.errors import YServiceError
from ydk.types import EntityCollection, Config
from ydk.entity_utils import _read_entities

class CRUDService(_CrudService):
    """
    Python wrapper for CrudService

    Provides API to Create, Read, Update, and Delete configuration on device.

    Args:
        provider (ydk.ServiceProvider): NetconfServiceProvider.
        entity_holder (ydk.types.Entity or a list(ydk.types.Entity)): Encoding target(s).
        pretty (bool, optional): Pretty formatting. Default value is True.

    Returns:
        Functions read and read-config return, ydk.types.Entity or a list(ydk.types.Entity), depending on provided input.
        Other functions return boolean value: true - for success; false - failure.

    Raises:
        Instance of YError in case of operation failure fails.
    """
    def __init__(self):
        self._crud = _CrudService()

    @_check_argument
    def create(self, provider, entity):
        if isinstance(entity, EntityCollection):
            entity = entity.entities()
        with _handle_error():
            return self._crud.create(provider, entity)

    def read(self, provider, read_filter=None):
        if provider is None:
            raise YServiceError("provider cannot be None")

        if read_filter is None:
            with _handle_error():
                return _read_entities(provider, get_config=False)

        filters = read_filter
        if isinstance(read_filter, EntityCollection):
            filters = read_filter.entities()
        with _handle_error():
            read_entity = self._crud.read(provider, filters)
        if isinstance(read_filter, EntityCollection):
            read_entity = Config(read_entity)
        return read_entity

    def read_config(self, provider, read_filter=None):
        if provider is None:
            raise YServiceError("provider cannot be None")

        if read_filter is None:
            with _handle_error():
                return _read_entities(provider)

        filters = read_filter
        if isinstance(read_filter, EntityCollection):
            filters = read_filter.entities()
        with _handle_error():
            read_entity = self._crud.read_config(provider, filters)
        if isinstance(read_filter, EntityCollection):
            read_entity = Config(read_entity)
        return read_entity

    @_check_argument
    def update(self, provider, entity):
        if isinstance(entity, EntityCollection):
            entity = entity.entities()
        with _handle_error():
            return self._crud.update(provider, entity)

    @_check_argument
    def delete(self, provider, entity):
        if isinstance(entity, EntityCollection):
            entity = entity.entities()
        with _handle_error():
            return self._crud.delete(provider, entity)



#  ----------------------------------------------------------------
# Copyright 2016 Cisco Systems
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ------------------------------------------------------------------

from .py_types import Entity, EntityCollection, Config, Filter, YList, YLeafList
from ydk.ext.types import Bits
from ydk.ext.types import ChildrenMap
from ydk.ext.types import ModelCachingOption
from ydk.ext.types import Decimal64
from ydk.ext.types import Empty
from ydk.ext.types import EncodingFormat
from ydk.ext.types import EntityPath
from ydk.ext.types import Enum
from ydk.ext.types import Identity
from ydk.ext.types import LeafData
from ydk.ext.types import LeafDataList
from ydk.ext.types import YType
from ydk.ext.types import YLeaf

__all__ = [ "YList",
            "Bits",
            "ChildrenMap",
            "Config",
            "ModelCachingOption",
            "Decimal64",
            "Empty",
            "EncodingFormat",
            "Entity",
            "EntityCollection",
            "EntityPath",
            "Enum",
            "Filter",
            "Identity",
            "LeafData",
            "LeafDataList",
            "YLeaf",
            "YLeafList",
            "YType",
            ]



#  ----------------------------------------------------------------
# Copyright 2016 Cisco Systems
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ------------------------------------------------------------------

import os
import logging
import importlib

from ydk.entity_utils import get_data_node_from_entity as _get_data_node_from_entity
from ydk.entity_utils import get_entity_from_data_node as _get_entity_from_data_node
from ydk.entity_utils import XmlSubtreeCodec
from ydk.entity_utils import _payload_to_top_entity, _get_bundle_name

from ydk.path import Codec as _Codec
from ydk.errors import YServiceProviderError as _YServiceProviderError
from ydk.errors import YServiceError as _YServiceError
from ydk.errors.error_handler import handle_runtime_error as _handle_error
from ydk.errors.error_handler import check_argument as _check_argument
from ydk.types import EncodingFormat


_TRACE_LEVEL_NUM = 5
_PAYLOAD_ERROR_MSG = "Codec service only supports one entity per payload, please split payload"


class CodecService(object):
    """CodecService wrapper.

    Attributes:
        logger (logging.Logger): CodecService logger.
    """

    def __init__(self):
        self.logger = logging.getLogger(__name__)

    @_check_argument
    def encode(self, provider, entity_holder, pretty=True, subtree=False):
        """Encode entities from entity_holder to string payload(s).

        Args:
            provider: An instance of ydk.provider.CodecServiceProvider class.
            entity_holder: an instance or collection of ydk.types.Entity class instances. 
                           Expected collection types - dict(str, ydk.types.Entity) or list(ydk.types.Entity).
            pretty: Pretty formatting of payload string, default - True.
            subtree: Bool flag, which specifies encode to XML subtree; default= False

        Returns:
            Payload in XML or JSON format based on configuration of Codec Provider.
            The type of return corresponds to the type of the 'entity_holder'.

        Raises:
            Instance of YError, if encoding fails.
        """
        if isinstance(entity_holder, dict):
            payload_map = {}
            for key in entity_holder:
                payload_map[key] = self._encode(provider, entity_holder[key], pretty, subtree)
            return payload_map
        elif isinstance(entity_holder, list):
            payload_list = []
            for entity in entity_holder:
                payload = self.encode(provider, entity, pretty, subtree)
                payload_list.append(payload)
            return payload_list
        else:
            return self._encode(provider, entity_holder, pretty, subtree)

    def _encode(self, provider, entity, pretty, subtree):
        """Encode a YDK entity to string payload.

        Args:
            provider (ydk.providers.CodecServiceProvider): Codec provider.
            entity (ydk.types.Entity) : Encoding target.
            pretty (bool): Pretty formatting if True.
            subtree: (bool) flag, which directs encode to XML subtree; default= False

        Returns:
            Encoded payload if success.

        Raises:
            Instance of YError is encoding fails.
        """
        bundle_name = _get_bundle_name(entity)
        provider.initialize(bundle_name, _get_yang_path(entity))
        root_schema = provider.get_root_schema(bundle_name)

        if subtree:
            if provider.encoding != EncodingFormat.XML:
                raise _YServiceError('Subtree option can only be used with XML encoding')
            xml_codec = XmlSubtreeCodec()
            return xml_codec.encode(entity, root_schema)

        with _handle_error():
            data_node = _get_data_node_from_entity(entity, root_schema)
            codec_service = _Codec()
            result = codec_service.encode(data_node, provider.encoding, pretty)
            self.logger.debug("Performing encode operation, resulting in {}".format(result))
            return result

    @_check_argument
    def decode(self, provider, payload_holder, subtree=False):
        """Decode payload in XML or JSON format to YDK entities.

        Args:
            provider: (ydk.providers.CodecServiceProvider): Codec provider.
            payload_holder: (str or dict(str, str) or list(str)), which represents payload in XML or JSON format.
            subtree: (bool) flag, which directs encode to XML subtree; default - False.

        Returns:
            An instance of ydk.types.Entity class or a dict(str, Entity) or list(Entity).

        Raises:
            YServiceProviderError, see documentation for `_decode`.
        """
        if isinstance(payload_holder, dict):
            entities = {}
            for key in payload_holder:
                entity = self.decode(provider, payload_holder[key], subtree)
                entities[key] = entity
            return entities
        elif isinstance(payload_holder, list):
            entities = []
            for payload in payload_holder:
                entity = self.decode(provider, payload, subtree)
                entities.append(entity)
            return entities
        else:
            return self._decode(provider, payload_holder, subtree)

    def _decode(self, provider, payload, subtree):
        """Decode payload to a YDK entity instance.

        Args:
            provider (ydk.providers.CodecServiceProvider): Codec provider.
            payload (str): Incoming payload, formatted in XML or JSON.

        Returns:
            A YDK entity (ydk.types.Entity) instance with children populated.

        Raises:
            - YServiceProviderError with _PAYLOAD_ERROR_MSG if payload
              contains more than one top level containers.
            - YServiceProviderError with _ENTITY_ERROR_MSG if no such entity
              could be found in locally installed YDK model packages.
        """
        entity = _payload_to_top_entity(payload, provider.encoding)

        if subtree:
            if provider.encoding != EncodingFormat.XML:
                raise _YServiceError('Subtree option can only be used with XML encoding')
            xml_codec = XmlSubtreeCodec()
            return xml_codec.decode(payload, entity)

        bundle_name = _get_bundle_name(entity)
        provider.initialize(bundle_name, _get_yang_path(entity))

        root_schema = provider.get_root_schema(bundle_name)

        self.logger.debug("Performing decode operation on payload:\n{}".format(payload))

        codec_service = _Codec()
        root_data_node = codec_service.decode(root_schema, payload, provider.encoding)
        data_nodes = root_data_node.get_children();
        if data_nodes is None or len(data_nodes) == 0:
            self.logger.debug(_PAYLOAD_ERROR_MSG)
            raise _YServiceProviderError(_PAYLOAD_ERROR_MSG)
        else:
            data_node = data_nodes[0]
            _get_entity_from_data_node(data_node, entity)
            return entity

    def _log_error_and_raise_exception(self, msg, exception_class):
        self.logger.error(msg)
        raise exception_class(msg)

def _get_yang_path(entity):
    """Return YANG models install location for entity.

    Args:
        entity (ydk.types.Entity): YDK entity instance.

    Returns:
        Path for installed YANG models location (str).
    """
    m = '.'.join(entity.__module__.rsplit('.')[0:3])
    m = importlib.import_module(m)
    return os.path.join(m.__path__[0], '_yang')



"""
    ydk.ext

    The code below is taken from Flask <http://flask.pocoo.org> project, and
    the original docstring is listed below.

    Redirect imports for extensions.  This module basically makes it possible
    for us to transition from flaskext.foo to flask_foo without having to
    force all extensions to upgrade at the same time.
    When a user does ``from flask.ext.foo import bar`` it will attempt to
    import ``from flask_foo import bar`` first and when that fails it will
    try to import ``from flaskext.foo import bar``.
    We're switching from namespace packages because it was just too painful for
    everybody involved.
    :copyright: (c) 2015 by Armin Ronacher.
    :license: BSD, see LICENSE for more details.

    :copyright: (c) 2015 by Armin Ronacher.
    :license: BSD, see LICENSE for more details.
"""
import atexit
import importlib


def setup():
    from ..exthook import ExtensionImporter
    importer = ExtensionImporter(['ydk_.%s'], __name__)
    importer.install()

def register_exit():

    def exit_handler():
        # allow sys.exit with logging for main block
        main = importlib.import_module('__main__')
        main.__dict__.clear()

    atexit.register(exit_handler)

register_exit()
setup()

del setup



#  ----------------------------------------------------------------
# Copyright 2017 Cisco Systems
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ------------------------------------------------------------------

"""restconf_session.py
RestconfSession Python wrapper.
"""

from ydk.ext.types import EncodingFormat as _EncodingFormat
from ydk.ext.path import RestconfSession as _RestconfSession


class RestconfSession(_RestconfSession):
    """ Python wrapper for RestconfSession.
    """

    def __init__(self, repo, address, username, password,
                       port=80, encoding=_EncodingFormat.JSON,
                       config_url_root="/data", state_url_root="/data"):
        super(RestconfSession, self).__init__(repo, address, username, password,
                                              port, encoding, config_url_root,
                                              state_url_root)

    def get_root_schema(self):
        return super(RestconfSession, self).get_root_schema()

    def invoke(self, rpc):
        return super(RestconfSession, self).invoke(rpc)



#  ----------------------------------------------------------------
# Copyright 2016 Cisco Systems
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ------------------------------------------------------------------

"""Setup for YDK
"""
from __future__ import print_function
import os
import subprocess
import sysconfig

from setuptools.command.build_ext import build_ext
from setuptools import setup, Extension, find_packages


NMSP_PKG_NAME = "$PACKAGE$"
NMSP_PKG_VERSION = "$VERSION$"
NMSP_PKG_DEPENDENCIES = ["$DEPENDENCY$"]

# Define and modify version number and package name here,
# Namespace packages are share same prefix: "ydk-models"
NAME = 'ydk'
VERSION = '0.7.2'
INSTALL_REQUIREMENTS = ['pybind11>=2.1.1']


LONG_DESCRIPTION = '''
                   The YANG Development Kit (YDK) is a Software Development Kit
                    that provides API's that are modeled in YANG. The main goal
                    of YDK is to reduce the learning curve of YANG data models by
                    expressing the model semantics in an API and abstracting
                    protocol/encoding details. YDK is composed of a core package
                    that defines services and providers, plus one or more module
                    bundles that are based on YANG models.
                   '''


YDK_PACKAGES = find_packages(exclude=['contrib', 'docs*', 'tests*',
                                      'ncclient', 'samples'])


class CMakeExtension(Extension):
    def __init__(self, name, sourcedir=''):
        Extension.__init__(self, name, sources=[])
        self.sourcedir = os.path.abspath(sourcedir)


class YdkBuildExtension(build_ext):
    def run(self):
        try:
            cmake3_installed = (
            0 == subprocess.call(['which', 'cmake3'], stdout=subprocess.PIPE, stderr=subprocess.PIPE))
            if not cmake3_installed:
                subprocess.check_output(['cmake', '--version'])
        except OSError:
            raise RuntimeError("CMake must be installed to build the following extensions: " +
                               ", ".join(e.name for e in self.extensions))

        for ext in self.extensions:
            self.build_extension(ext)

    def build_extension(self, ext):
        try:
            import pybind11
        except ImportError:
            import pip
            pip.main(['install', 'pybind11>=2.1.1'])
            import pybind11

        extdir = os.path.abspath(os.path.dirname(self.get_ext_fullpath(ext.name)))
        coverage_compiler_flag = '-DCOVERAGE=False'
        if 'YDK_COVERAGE' in os.environ:
            coverage_compiler_flag = '-DCOVERAGE=True'
        cmake_args = ['-DCMAKE_LIBRARY_OUTPUT_DIRECTORY={0}'.format(extdir),
                      '-DPYBIND11_INCLUDE={0};{1}'.format(
                                      pybind11.get_include(),
                                      pybind11.get_include(user=True)),
                      '-DPYTHON_VERSION={0}'.format(
                                      get_python_version()),
                      '-DCMAKE_BUILD_TYPE=Release',
                      coverage_compiler_flag]

        if not os.path.exists(self.build_temp):
            os.makedirs(self.build_temp)

        cmake3_installed = (0 == subprocess.call(['which', 'cmake3'], stdout=subprocess.PIPE, stderr=subprocess.PIPE))
        if(cmake3_installed):
            cmake_executable = 'cmake3'
        else:
            cmake_executable = 'cmake'

        subprocess.check_call([cmake_executable, ext.sourcedir] + cmake_args, cwd=self.build_temp)
        subprocess.check_call([cmake_executable, '--build', '.'], cwd=self.build_temp)


def get_python_version():
    python_version = sysconfig.get_config_var('LDVERSION')
    if python_version is None or len(python_version) == 0:
        python_version = sysconfig.get_config_var('VERSION')
    return python_version


setup(
    name=NAME,
    version=VERSION,
    description='YDK Python SDK',
    long_description=LONG_DESCRIPTION,
    url='https://github.com/CiscoDevNet/ydk-py',
    author='Cisco Systems',
    author_email='yang-dk@cisco.com',
    license='Apache 2.0',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Intended Audience :: Education',
        'Intended Audience :: Information Technology',
        'Intended Audience :: Science/Research',
        'Intended Audience :: System Administrators',
        'Topic :: Software Development :: Build Tools',
        'Topic :: Software Development :: Libraries',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: C++'
    ],
    keywords='yang, C++11, python bindings ',
    packages=YDK_PACKAGES,
    install_requires=INSTALL_REQUIREMENTS,
    ext_modules=[CMakeExtension('ydk_')],
    cmdclass={
             'build_ext' :YdkBuildExtension
             },
    zip_safe=False,
)
